package cn.flyrise.pai.xlink.exception;

import cn.flyrise.starter.util.I18nUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 业务错误代码定义枚举类，错误代码 1001-4999
 * <br />
 * 放于Service层在处理逻辑时根据判断抛出相应异常，ex:
 * <pre>
 *       public boolean insert(UserPO userPO) {
 *         if (StrUtil.isNotEmpty(userPO.getId()) && userMapper.selectById(userPO.getId()) != null) {
 *             throw new DemoBizException(BusinessErrors.DUPLICATE_KEY);
 *         }
 *
 *         return SqlHelper.retBool(userMapper.insert(userPO));
 *     }
 * </pre>
 *
 * @author pai
 * @date 2023-02-24 09:28:09
 */
public enum BusinessErrors {
    //手动指定错误码
    ERR_1001("demo", "数据已存在"),

    //... 其他错误码的定义
    ERR_4000("auth", "账号信息未同步，请稍后再试"),
    ERR_5000("sync", "找不到同步的平台:{}"),
    ERR_5001("sync","未开启同步，请确认参数是否正确"),
    ERR_5002("logout","返回门户失败")
    ;

    /**
     * 错误代码 1001-4999
     */
    private String code;
    /**
     * 错误消息
     */
    private String msg;
    /**
     * 错误消息国际化key
     */
    private String key;


    /**
     * ERR_nnnn 的命名方式使用
     *
     * @param module 模块代码
     * @param msg    内容
     */
    BusinessErrors(String module, String msg) {
        this.msg = msg;
        String name = this.name();
        if (!name.contains("_")) {
            throw new RuntimeException("命名不符合规范, 如：ERR_1001、 MSG_1001、 VIEW_5001");
        }
        String[] names = name.split("_");
        if (!ReUtil.isMatch("^[a-zA-Z]+$", names[0]) || !NumberUtil.isInteger(names[1])) {
            throw new RuntimeException("命名不符合规范, 如：ERR_1001、 MSG_1001、 VIEW_5001");
        }
        name = names[0];
        this.code = names[1];
        if ("ERR".equals(name)) {
            name = "error";
        } else if ("MSG".equals(name)) {
            name = "message";
        } else {
            name = name.toLowerCase();
        }
        this.key = StrUtil.format("{}.{}.{}", name, module, this.code);
    }

    /**
     * 自定义完整内容
     *
     * @param code 错误代码
     * @param msg  内容
     * @param key  配置文件Key
     */
    BusinessErrors(String code, String msg, String key) {
        this.code = code;
        this.msg = msg;
        this.key = key;
    }

    public String getCode() {
        return code;
    }

    public String getMsg() {
        String msgStr = I18nUtil.get(key, msg);
        if (key != null && !key.equals(msgStr)) {
            return msgStr;
        }
        return msg;
    }

    public String getKey() {
        return key;
    }

    /**
     * 获取参数化的msg值
     */
    public String format(Object... params) {
        String msg = this.getMsg();
        if (msg != null && msg.contains(StrUtil.EMPTY_JSON)) {
            return StrUtil.format(msg, params);
        }
        return msg;
    }

    /*public static void main(String[] args) {
        java.util.List<String> msgs = new java.util.ArrayList<>();
        java.util.Optional.ofNullable(cn.hutool.core.util.ClassUtil.scanPackage("cn.flyrise.pai"))
                .ifPresent(classes -> {
                    for (Class<?> clazz : classes) {
                        java.util.Optional.ofNullable(cn.hutool.core.util.ClassUtil.getDeclaredFields(clazz))
                                .ifPresent(fields -> {
                                    for (java.lang.reflect.Field field : fields) {
                                        java.util.Optional.ofNullable(field
                                                .getAnnotation(cn.flyrise.common.core.domain.InvalidCode.class))
                                                .ifPresent(invalidCode -> {
                                                    String message = invalidCode.message();
                                                    if (StrUtil.isBlank(message)) {
                                                        message = "invalid.code.".concat(invalidCode.value());
                                                    } else if(message.startsWith("{") && message.endsWith("}")){
                                                        message = message.substring(1, message.length() - 1);
                                                    }
                                                    msgs.add(message.concat("=").concat(invalidCode.desc()));
                                                });
                                    }
                                });
                    }
                });
        msgs.stream().sorted().collect(java.util.stream.Collectors.toCollection(java.util.LinkedHashSet::new))
                .forEach(System.out::println);
        for (BusinessErrors value : BusinessErrors.values()) {
            System.out.println(value.getKey() + "=" + value.getMsg());
        }
    }*/
}
